;Author: Qu
;Version 1.00. First public release.

#NoEnv
SendMode Input
SetWorkingDir %A_ScriptDir%
SetDefaultMouseSpeed, 0
SetMouseDelay, -1
SetTitleMatchMode, 3        ;Exact window title match
ATITDWindow:="eGenesis Client ahk_class eGenesis Client"
ConfigFile:="QGlassConfig.ini"

MaxTempError:=25       ;The next seven variables are used to estimate spikes
TeppyTimeFactor:=1.11  ;
MaxDV:=170             ;Your highest DV bench. 
MaxHV:=60              ;
MinDV:=100
MinHV:=40
SafetyMargin:=10       ;Add this many extra seconds before spiking

SpikeTemp:=1900        ;Will spike when below this temp
MaxAddTemp:=2400 - MaxTempError - MaxHV ;Will add 2 cc when below this temp

;Zero out starting variables
Benches:=0

;Define constants
SODA_GLASS:=1
NORMAL_GLASS:=2
JEWEL_GLASS:=3
NO_GLASS:=4

;Edit these if for some reason the macro can't read the config file
DefaultSodaGlassFile:="glass_rod.png"
DefaultNormalGlassFile:="sheet_of_glass.png"
DefaultJewelGlassFile:="fine_glass_rod.png"

LoadConfig()
If (ErrorLevel != 0)
{
  MsgBox, Error reading config file: %ConfigFile%. Using defaults.
}

;Find game window(s)
While (WindowHandle == "" || WindowHandle == 0)
{
  WinWait, %ATITDWindow%
  WinGet, WindowHandle, List, %ATITDWindow%
  If (A_Index > 50)
  {
    MsgBox, Error detecting ATITD Window.
    return
  }
}
If (%WindowHandle% > 1)
{
  MsgBox, Multiple game windows found. Multiple clients not supported.
}
WinGetPos, WinPosX, WinPosY, WindowWidth, WindowHeight, ahk_id %WindowHandle1%

;Here we find all the glazier bench windows
BenchFilename1:="this_is_your_glaziers_bench.png"
BenchFilename2:="this_is_a_glaziers_bench.png"
Loop, 2
{
  SearchStartX1:=0
  SearchStartY1:=0
  BenchFilename:=BenchFilename%A_Index%
  While (SearchStartY1 <= WindowHeight - 150)
  {
    WinWaitActive, %ATITDWindow%
    ImageSearch, FoundX1, FoundY1, SearchStartX1, SearchStartY1, WindowWidth, WindowHeight - 150, *60 %BenchFilename%
    If (ErrorLevel == 2)
    {
      MsgBox, %BenchFilename%: File not found or invalid search area.
      return
    }
    Else If (ErrorLevel == 1)
    {
      break
    }
    Else If (ErrorLevel == 0)
    {
      Benches++
      BenchX%Benches%:=FoundX1
      BenchY%Benches%:=FoundY1
      SearchStartY1:=FoundY1 + 1
    
      ;Now we find all benches hiding on the same row
      SearchStartX2:=FoundX1 + 1
      SearchStartY2:=FoundY1
      While (SearchStartX2 <= WindowWidth)
      {
        ImageSearch, FoundX2, FoundY2, SearchStartX2, SearchStartY2, WindowWidth, SearchStartY2 + 25, *60 %BenchFilename%
        If (ErrorLevel == 2)
        {
          MsgBox, %BenchFilename%: File not found or invalid search area.
          return
        }
        Else If (ErrorLevel == 1)
        {
          break
        }
        Else If (ErrorLevel == 0)
        {
          Benches++
          BenchX%Benches%:=FoundX2
          BenchY%Benches%:=FoundY2
          SearchStartX2:=FoundX2 + 1
        }
        Else
        {
          MsgBox, Uknown ErrorLevel: %ErrorLevel%
          return
        }
      }
    }
    Else
    {
      MsgBox, Uknown ErrorLevel: %ErrorLevel%
      return
    }
  }
}

;Now we determine the dimensions of each window
While ((i:=A_Index) <= Benches)
{
  ImageSearch, UpperRightX, , BenchX%i%, BenchY%i%, WindowWidth, BenchY%i% + 7, *60 upper_right_corner.png
  If (ErrorLevel == 2)
  {
    MsgBox, upper_right_corner.png: File not found or invalid search parameters.
    return
  }
  Else If (ErrorLevel == 1)
  {
    MsgBox, Could not find the upper right corner of bench %i%. Make sure the windows aren't overlapping. Aborting.
    return
  }
  Else If (ErrorLevel == 0)
  {
    ;We know the window width now, so let's find the height as well
    BenchEndX%i%:=UpperRightX + 8

    ImageSearch, , LowerRightY, BenchEndX%i% - 16, BenchY%i%, BenchEndX%i% - 1, WindowHeight, *60 lower_right_corner.png
    If (ErrorLevel == 2)
    {
      MsgBox, lower_right_corner.png: File not found or invalid search parameters.
      return
    }
    Else If (ErrorLevel == 1)
    {
      MsgBox, Could not find the lower right corner of bench %i%. Make sure the windows aren't overlapping. Aborting.
      return
    }
    Else If (ErrorLevel == 0)
    {
      BenchEndY%i%:=LowerRightY + 16
    }
    Else
    {
      MsgBox, Uknown ErrorLevel: %ErrorLevel%
      return
    }
  }
  Else
  {
    MsgBox, Uknown ErrorLevel: %ErrorLevel%
    return
  }
}

;Now that we know the exact dimensions of each window, we identify all of their important features.
While ((i:=A_Index) <= Benches)
{
  ImageSearch, FoundX, FoundY, BenchX%i%, BenchY%i%, BenchEndX%i%, BenchEndY%i%, *60 temperature_word.png
  If (ErrorLevel == 2)
  {
    MsgBox, temperature_word.png: File not found or invalid search parameters.
    return
  }
  Else If (ErrorLevel == 1)
  {
    MsgBox, temperature_word.png: not found.
    return
  }
  Else If (ErrorLevel == 0)
  {
    BenchTempY%i%:=FoundY
    BenchGlassY%i%:=FoundY + 20
    BenchCharcoalY%i%:=FoundY + 40
    BenchMakingY%i%:=FoundY + 60
    BenchTempBarY%i%:=FoundY - 25
  }
  Else
  {
    MsgBox, Uknown ErrorLevel: %ErrorLevel%
    return
  }
  
  ImageSearch, LeftSideX, , BenchX%i%, BenchY%i%, BenchEndX%i%, BenchEndY%i%, *60 left_side.png
  If (ErrorLevel == 2)
  {
    MsgBox, left_side.png: File not found or invalid search parameters.
    return
  }
  Else If (ErrorLevel == 1)
  {
    MsgBox, left_side.png: not found.
    return
  }
  Else If (ErrorLevel == 0)
  {
    BenchTempBarX%i%:=LeftSideX
  }
  Else
  {
    MsgBox, Uknown ErrorLevel: %ErrorLevel%
    return
  }

  ImageSearch, RightSideX, , BenchX%i%, BenchY%i%, BenchEndX%i%, BenchEndY%i%, *60 right_side.png
  If (ErrorLevel == 2)
  {
    MsgBox, right_side.png: File not found or invalid search parameters.
    return
  }
  Else If (ErrorLevel == 1)
  {
    MsgBox, right_side.png: not found.
    return
  }
  Else If (ErrorLevel == 0)
  {
    BenchTempBarEndX%i%:=RightSideX + 3
    BenchTempBarLength%i%:=BenchTempBarEndX%i% - BenchTempBarX%i%
  }
  Else
  {
    MsgBox, Uknown ErrorLevel: %ErrorLevel%
    return
  }
  
  ImageSearch, , , BenchX%i%, BenchGlassY%i%, BenchEndX%i%, BenchGlassY%i% + 10, *60 soda_glass.png
  If (ErrorLevel == 0)
  {
    BenchGlassType%i%:=SODA_GLASS
  }
  ImageSearch, , , BenchX%i%, BenchGlassY%i%, BenchEndX%i%, BenchGlassY%i% + 10, *60 normal_glass.png
  If (ErrorLevel == 0)
  {
    BenchGlassType%i%:=NORMAL_GLASS
  }
  ImageSearch, , , BenchX%i%, BenchGlassY%i%, BenchEndX%i%, BenchGlassY%i% + 10, *60 jewel_glass.png
  If (ErrorLevel == 0)
  {
    BenchGlassType%i%:=JEWEL_GLASS
  }
  ImageSearch, , , BenchX%i%, BenchGlassY%i%, BenchEndX%i%, BenchGlassY%i% + 10, *60 no_melted_glass.png
  If (ErrorLevel == 0)
  {
    BenchGlassType%i%:=NO_GLASS
  }
  If (BenchGlassType%i% == "")
  {
    MsgBox, Could not determine glass type of bench %i%.
    return
  }
}

If (Benches != "" && Benches > 0)
{
  MsgBox, Found %Benches% Glazier's Benches. Ready to start.
}
Else
{
  MsgBox, No Glazier's Benches found.
}

;Hotkeys will only work in ATITD
#IfWinActive, eGenesis Client ahk_class eGenesis Client

;Main loop
;SpikeStage at 0 means we saw the temperature drop and we're free to spike
;SpikeStage at 1 means we're waiting to add 2 cc to spike
;SpikeStage at 2 means we've spiked and waiting for temperature to jump
^!a::
While (Benches > 0)
{
  ;i cycles from 1 up to and including Benches, then rolls back over to 1
  i:=(mod(A_Index, Benches) == 0 ? Benches : mod(A_Index, Benches))
  
  ;Skip the bench if it's disabled or temporarily locked
  If (BenchDisabled%i% == 1 || Bench%i%LockedUntil > A_TickCount)
  {
    continue
  }

  PrevTemp:=BenchTemp%i%
  BenchTemp%i%:=ReadTempBar(i)
  DeltaTemp:=BenchTemp%i% - PrevTemp

  If (BenchTemp%i% == "" || BenchTemp%i% < 0)
  {
    ;Something went wrong with temp reading. Too dangerous to try anything. Skip this bench.
    ;TODO: Could try the alternate method of getting temp by reading the numbers
    continue
  }

  If (Bench%i%SpikeStage == 1 && A_TickCount > Bench%i%SpikeAt)
  {
    AddCC(i, 2)
    Bench%i%SpikeStage:=2
    Bench%i%LockedUntil:=A_TickCount + 2000 ;Lock for 2 seconds
    continue
  }

  ;Keep adding 2 cc every time temp drops
  If (DeltaTemp != "" && DeltaTemp < 0 && BenchTemp%i% < MaxAddTemp)
  {
    AddCC(i, 2)
    Bench%i%LockedUntil:=A_TickCount + 2000 ;Lock for 2 seconds
    Bench%i%SpikeStage:=0
    If (BenchTemp%i% < SpikeTemp)
    {
      WaitTime:=TeppyTimeFactor*(90 + SafetyMargin + 10/MaxDV*(MaxTempError + BenchTemp%i% + 2*MaxHV - 2400))
      WaitTime:=(WaitTime < 15 ? 15 : WaitTime)
      Bench%i%SpikeAt:=A_TickCount + Floor(1000*WaitTime)
      Bench%i%SpikeStage:=1
    }
    continue
  }
  
  ;Here we see if we can make the item
  GlassFile:=(BenchGlassType%i% == SODA_GLASS ? SodaGlassFile : "") . (BenchGlassType%i% == NORMAL_GLASS ? NormalGlassFile : "") . (BenchGlassType%i% == JEWEL_GLASS ? JewelGlassFile : "")
  If (BenchTemp%i% > 1600 + MaxDV + MaxTempError && BenchTemp%i% < 2400 && IsBenchReady(i) == 1)
  {
    FullState:=IsBenchFull(i)
    If (FullState == 0)
    {
      BenchDisabled%i%:=1
      continue
    }
    ImageSearch, FoundX, FoundY, BenchX%i%, BenchY%i%, BenchEndX%i%, BenchTempBarY%i%, *60 %GlassFile%
    If (ErrorLevel == 0 && FullState == 1)
    {
      LoadConfig()
      KeyWait, LButton
      KeyWait, RButton
      BlockInput, MouseMove
      MouseGetPos, MouseX, MouseY
      BlockInput, Mouse
      MouseClick, Left, FoundX, FoundY
      MouseMove, MouseX, MouseY
      BlockInput, MouseMoveOff
      Bench%i%LockedUntil:=A_TickCount + 3000
    }
    If (ErrorLevel == 1 && FullState == 1)
    {
      KeyWait, LButton
      KeyWait, RButton
      BlockInput, MouseMove
      MouseGetPos, MouseX, MouseY
      BlockInput, Mouse
      MouseClick, Left, BenchX%i%, BenchY%i%
      MouseMove, MouseX, MouseY
      BlockInput, MouseMoveOff
      Bench%i%LockedUntil:=A_TickCount + 3000
    }
  }
}
return

;Toggles the disabled/enabled state of a bench
^#d::
CurrentBench:=MouseOverBench()
If (CurrentBench == "")
{
  SoundPlay, *-1
  return
}
If (BenchDisabled%CurrentBench% != 1)
{
  BenchDisabled%CurrentBench%:=1
  ToolTipIndex:=(CurrentBench > 20 ? 20 : CurrentBench)
  ToolTip, ***DISABLED***, BenchX%CurrentBench% + 10, BenchY%CurrentBench%, ToolTipIndex
}
Else
{
  BenchDisabled%CurrentBench%:=0
  ToolTipIndex:=(CurrentBench > 20 ? 20 : CurrentBench)
  ToolTip, Enabled, BenchX%CurrentBench% + 10, BenchY%CurrentBench%, ToolTipIndex
}
return

;Returns the bench number of the moused over bench
;Returns null if not over a bench
MouseOverBench()
{
  local MouseX, MouseY, i
  MouseGetPos, MouseX, MouseY
  While ((i:=A_Index) <= Benches)
  {
    If (MouseX >= BenchX%i% && MouseX < BenchEndX%i% && MouseY >= BenchY%i% && MouseY < BenchEndY%i%)
    {
      return i
    }
  }
}

;Returns 1 if there's more than 19 glass in the bench
;Returns 0 if there's less than 20 glass in the bench
;Returns -1 on error
IsBenchFull(Bench)
{
  local GlassX, GlassY, Digit1X, Digit2X
  
  If(Bench == "" || Bench <= 0)
  {
    return -1
  }
  
  ImageSearch, GlassX, GlassY, BenchTempBarX%Bench%, BenchGlassY%Bench%, BenchTempBarEndX%Bench%, BenchGlassY%Bench% + 8, *60 glass_colon.png
  If (ErrorLevel == 2)
  {
    return -1
  }
  Else If (ErrorLevel == 1)
  {
    ;no_melted_glass.png is 84x11
    ImageSearch, , , BenchTempBarX%Bench%, BenchGlassY%Bench%, BenchTempBarEndX%Bench%, BenchGlassY%Bench% + 10, *60 no_melted_glass.png
    If (ErrorLevel == 0)
    {
      return 0
    }
    Else
    {
      return -1
    }
  }
  Digit1X:=GlassX + 29 + 6
  Digit2X:=GlassX + 29 + 6 + 7

  ImageSearch, , , Digit1X, BenchGlassY%Bench%, Digit1X + 5, BenchGlassY%Bench% + 8, *60 1.png
  If (ErrorLevel == 0)
  {
    return 0
  }
  Else If (ErrorLevel != 1)
  {
    return -1
  }
  
  ;Ok, so the first digit isn't 1. But if there's a 2nd digit, we'll know that the bench has more than 19 glass.
  ImageSearch, , , Digit2X, BenchGlassY%Bench%, Digit2X + 5, BenchGlassY%Bench% + 8, *60 empty_digit.png
  If (ErrorLevel == 0)
  {
    return 0
  }
  Else If (ErrorLevel != 1)
  {
    return -1
  }
  return 1
}

;Returns 1 if the bench isn't currently making anything
;Returns 0 if the bench is making something
;Returns -1 on error
IsBenchReady(Bench)
{
  local Length, StartX
  
  If (Bench == "" || Bench <= 0)
  {
    return -1
  }
  
  ;Length:=BenchTempBarEndX%Bench% - BenchTempBarX%Bench%
  ;StartX:=(Length - 120)/2 + BenchTempBarX%Bench%
  StartX:=BenchTempBarEndX%Bench%/2 + BenchTempBarX%Bench%/2 - 60
  ImageSearch, , , StartX, BenchMakingY%Bench%, StartX + 119, BenchMakingY%Bench% + 8, *60 empty_120x9.png
  If (ErrorLevel == 1)
  {
    return 0
  }
  Else If (ErrorLevel == 0)
  {
    return 1
  }
  return -1
}

;Returns 0 on successful click
;Returns 1 on failure
AddCC(Bench, Amount=2)
{
  local FoundX, FoundY, MouseX, MouseY
  
  If (Bench == "" || Bench <= 0)
  {
    return 1
  }
  ImageSearch, FoundX, FoundY, BenchX%Bench%, BenchY%Bench%, BenchEndX%Bench%, BenchTempY%Bench%, *60 add_%Amount%_charcoal.png
  If (ErrorLevel == 0) 
  {
    KeyWait, LButton
    KeyWait, RButton
    BlockInput, MouseMove
    MouseGetPos, MouseX, MouseY
    BlockInput, Mouse
    MouseClick, Left, FoundX, FoundY
    MouseMove, MouseX, MouseY
    BlockInput, MouseMoveOff
    return 0
  }
  return 1
}

;Sets ErrorLevel to 0 on success and 1 on failure
LoadConfig()
{
  local Type1, Type2, Type3, GlassType
  
  Type1:="Soda"
  Type2:="Normal"
  Type3:="Jewel"
  
  ErrorLevel:=0
  Loop, 3
  {
    GlassType:=Type%A_Index%
    IniRead, %GlassType%GlassFile, %ConfigFile%, Products, %GlassType%, %A_Space%
    If (%GlassType%GlassFile == "")
    {
      %GlassType%GlassFile:=Default%GlassType%GlassFile
      ErrorLevel:=1
    }
  }
}

;Returns estimated temperature
;Estimate error is something like 2500/BarLength
;Returns -1 if temperature could not be determined
;Returns -2 if there was an error conducting ImageSearch
ReadTempBar(Bench)
{
  local FoundX
  local BarLength:=BenchTempBarEndX%Bench% - BenchTempBarX%Bench%
  
  ImageSearch, FoundX, , BenchTempBarX%Bench%, BenchTempBarY%Bench%, BenchTempBarEndX%Bench%, BenchTempBarY%Bench% + 17, *60 temp_bar_end.png
  If (ErrorLevel == 2)
  {
    return -2
  }
  If (ErrorLevel == 1)
  {
    ;Unreliable. This whole part needs to be rewritten
    ImageSearch, , , BenchTempBarX%Bench%, BenchTempBarY%Bench%, BenchTempBarX%Bench% + 1, BenchTempBarY%Bench% + 17, *10 temp_bar_2pixel.png
    If (ErrorLevel == 0)
    {
      return Floor(2*5000/BarLength)
    }

    ImageSearch, , , BenchTempBarX%Bench%, BenchTempBarY%Bench%, BenchTempBarX%Bench%, BenchTempBarY%Bench% + 17, *10 temp_bar_1pixel.png
    If (ErrorLevel == 0)
    {
      return Floor(5000/BarLength)
    }

    ImageSearch, , , BenchTempBarX%Bench%, BenchTempBarY%Bench%, BenchTempBarEndX%Bench%, BenchTempBarY%Bench% + 17, *60 temp_bar_empty.png
    If (ErrorLevel == 0)
    {
      return 0
    }

    ImageSearch, , , BenchTempBarX%Bench%, BenchTempBarY%Bench%, BenchTempBarEndX%Bench%, BenchTempBarY%Bench% + 17, *60 full_temp_bar.png
    If (ErrorLevel == 0)
    {
      return 5000
    }

    return -1
  }
  If (ErrorLevel == 0)
  {
    return Floor((FoundX - BenchTempBarX%Bench% + 1)*5000/BarLength)
  }
}

#IfWinActive
;SCRIPT HOTKEYS

^+!`::Pause
^+!r::Reload
^+!e::Edit